/*

SDL2_gfxPrimitives.h: graphics primitives for SDL

Copyright (C) 2012  Andreas Schiffler

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.

Andreas Schiffler -- aschiffler at ferzkopp dot net

*/

/*
  Ported to OpenEuphoria 4 by Jean-Marc DURO

  v 0.1 - 2016-01-08
  Basic primitives only

  v 0.2 - 2016-01-08
  All functions wrapped

  Documentation available at http://www.ferzkopp.net/Software/SDL_gfx-2.0/Docs/html/index.html

*/

include std/dll.e
include std/machine.e
include std/os.e
include EuSDL2.e
public include SDL2_gfxPrimitives_font.e

atom sdl2_gfx

ifdef WINDOWS then
  sdl2_gfx = open_dll("../Bin/SDL2_gfx.dll")
elsifdef UNIX then
  sequence un = uname()
  sequence arch = un[5]
  -- following lines work for Ubuntu derivatives
  -- with other Linux distributions please update corresponding lines
  if equal(arch, "i686") then
    sdl2_gfx = open_dll("/usr/lib/i386-linux-gnu/libSDL2_gfx.so")
  elsif equal(arch, "x86_64") then
    sdl2_gfx = open_dll("/usr/lib/x86_64-linux-gnu/libSDL2_gfx.so")
  end if
end ifdef
if sdl2_gfx = -1 then
    puts(1,"Could not open SDL2_gfx library!\n")
    abort(0)
end if


constant M_PI = 3.1415926535897932384626433832795

constant
  SDL2_GFXPRIMITIVES_MAJOR = 1,
  SDL2_GFXPRIMITIVES_MINOR = 0,
  SDL2_GFXPRIMITIVES_MICRO = 1

public constant
  SMOOTHING_OFF = 0,  -- Disable anti-aliasing (no smoothing)
  SMOOTHING_ON  = 1   -- Enable anti-aliasing (smoothing)

public constant
  FPS_UPPER_LIMIT = 200,  -- Highest possible rate supported by framerate controller in Hz (1/s)
  FPS_LOWER_LIMIT = 1,    -- Lowest possible rate supported by framerate controller in Hz (1/s)
  FPS_DEFAULT     = 30    -- Default rate of framerate controller in Hz (1/s)

/*
  -- Structure holding the state and timing information of the framerate controller.
  typedef struct {
    Uint32 framecount;
    float rateticks;
    Uint32 baseticks;
    Uint32 lastticks;
    Uint32 rate;
  } FPSmanager;
*/

public function GFX_Version()
  return sprintf("%d.%d.%d", {SDL2_GFXPRIMITIVES_MAJOR,
                 SDL2_GFXPRIMITIVES_MINOR, SDL2_GFXPRIMITIVES_MICRO})
end function

  /* Note: all ___Color routines expect the color to be in format 0xRRGGBBAA */

public constant
  /* Pixel */

  xpixelColor = define_c_func(sdl2_gfx, "pixelColor", {C_POINTER, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xpixelRGBA  = define_c_func(sdl2_gfx, "pixelRGBA", {C_POINTER, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Horizontal line */

  xhlineColor = define_c_func(sdl2_gfx, "hlineColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xhlineRGBA  = define_c_func(sdl2_gfx, "hlineRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Vertical line */

  xvlineColor = define_c_func(sdl2_gfx, "vlineColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xvlineRGBA  = define_c_func(sdl2_gfx, "vlineRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Rectangle */

  xrectangleColor = define_c_func(sdl2_gfx, "rectangleColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_UINT}, C_INT),
  xrectangleRGBA  = define_c_func(sdl2_gfx, "rectangleRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_UBYTE, C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Rounded-Corner Rectangle */

  xroundedRectangleColor = define_c_func(sdl2_gfx, "roundedRectangleColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT,
                                                   C_SHORT, C_SHORT, C_UINT}, C_INT),
  xroundedRectangleRGBA  = define_c_func(sdl2_gfx, "roundedRectangleRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT,
                                                   C_SHORT, C_SHORT, C_UBYTE, C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Filled rectangle (Box) */

  xboxColor = define_c_func(sdl2_gfx, "boxColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xboxRGBA  = define_c_func(sdl2_gfx, "boxRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Rounded-Corner Filled rectangle (Box) */

  xroundedBoxColor = define_c_func(sdl2_gfx, "roundedBoxColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_SHORT, C_UINT}, C_INT),
  xroundedBoxRGBA  = define_c_func(sdl2_gfx, "roundedBoxRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_SHORT, C_UBYTE, C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Line */

  xlineColor = define_c_func(sdl2_gfx, "lineColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xlineRGBA  = define_c_func(sdl2_gfx, "lineRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* AA Line */

  xaalineColor = define_c_func(sdl2_gfx, "aalineColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xaalineRGBA  = define_c_func(sdl2_gfx, "aalineRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Thick Line */
  xthickLineColor = define_c_func(sdl2_gfx, "thickLineColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE,
                                                   C_UINT}, C_INT),
  xthickLineRGBA  = define_c_func(sdl2_gfx, "thickLineRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE,
                                                   C_UBYTE, C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Circle */

  xcircleColor = define_c_func(sdl2_gfx, "circleColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xcircleRGBA  = define_c_func(sdl2_gfx, "circleRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Arc */

  xarcColor = define_c_func(sdl2_gfx, "arcColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xarcRGBA  = define_c_func(sdl2_gfx, "arcRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* AA Circle */

  xaacircleColor = define_c_func(sdl2_gfx, "aacircleColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xaacircleRGBA  = define_c_func(sdl2_gfx, "aacircleRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE, C_UBYTE,
                                                   C_UBYTE}, C_INT),

  /* Filled Circle */

  xfilledCircleColor = define_c_func(sdl2_gfx, "filledCircleColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xfilledCircleRGBA  = define_c_func(sdl2_gfx, "filledCircleRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Ellipse */

  xellipseColor = define_c_func(sdl2_gfx, "ellipseColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xellipseRGBA  = define_c_func(sdl2_gfx, "ellipseRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* AA Ellipse */

  xaaellipseColor = define_c_func(sdl2_gfx, "aaellipseColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xaaellipseRGBA  = define_c_func(sdl2_gfx, "aaellipseRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Filled Ellipse */

  xfilledEllipseColor = define_c_func(sdl2_gfx, "filledEllipseColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT},
                                                   C_INT),
  xfilledEllipseRGBA  = define_c_func(sdl2_gfx, "filledEllipseRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE,
                                                   C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Pie */

  xpieColor = define_c_func(sdl2_gfx, "pieColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT}, C_INT),
  xpieRGBA  = define_c_func(sdl2_gfx, "pieRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Filled Pie */

  xfilledPieColor = define_c_func(sdl2_gfx, "filledPieColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UINT},
                                                   C_INT),
  xfilledPieRGBA  = define_c_func(sdl2_gfx, "filledPieRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_UBYTE,
                                                   C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Trigon */

  xtrigonColor = define_c_func(sdl2_gfx, "trigonColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_UINT}, C_INT),
  xtrigonRGBA  = define_c_func(sdl2_gfx, "trigonRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_UBYTE, C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* AA-Trigon */

  xaatrigonColor = define_c_func(sdl2_gfx, "aatrigonColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_UINT}, C_INT),
  xaatrigonRGBA  = define_c_func(sdl2_gfx, "aatrigonRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_UBYTE, C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Filled Trigon */

  xfilledTrigonColor = define_c_func(sdl2_gfx, "filledTrigonColor", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_SHORT, C_UINT}, C_INT),
  xfilledTrigonRGBA  = define_c_func(sdl2_gfx, "filledTrigonRGBA", {C_POINTER, C_SHORT, C_SHORT, C_SHORT, C_SHORT, C_SHORT,
                                                   C_SHORT, C_UBYTE, C_UBYTE, C_UBYTE, C_UBYTE}, C_INT),

  /* Polygon */

  xpolygonColor = define_c_func(sdl2_gfx, "polygonColor", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_UINT}, C_INT),
  xpolygonRGBA  = define_c_func(sdl2_gfx, "polygonRGBA", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_UBYTE, C_UBYTE, C_UBYTE,
                                                   C_UBYTE}, C_INT),

  /* AA-Polygon */

  xaapolygonColor = define_c_func(sdl2_gfx, "aapolygonColor", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_UINT}, C_INT),
  xaapolygonRGBA = define_c_func(sdl2_gfx, "aapolygonRGBA", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_UBYTE, C_UBYTE, C_UBYTE,
                                                   C_UBYTE}, C_INT),

  /* Filled Polygon */

  xfilledPolygonColor = define_c_func(sdl2_gfx, "filledPolygonColor", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_UINT}, C_INT),
  xfilledPolygonRGBA  = define_c_func(sdl2_gfx, "filledPolygonRGBA", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Textured Polygon */

  xtexturedPolygon = define_c_func(sdl2_gfx, "texturedPolygon", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_POINTER, C_INT,
                                                   C_INT}, C_INT),

  /* Bezier */

  xbezierColor = define_c_func(sdl2_gfx, "bezierColor", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_INT, C_UINT}, C_INT),
  xbezierRGBA  = define_c_func(sdl2_gfx, "bezierRGBA", {C_POINTER, C_POINTER, C_POINTER, C_INT, C_INT, C_UBYTE, C_UBYTE,
                                                   C_UBYTE, C_UBYTE}, C_INT),

  /* Characters/Strings */

  xgfxPrimitivesSetFont = define_c_proc(sdl2_gfx, "gfxPrimitivesSetFont", {C_POINTER, C_UINT, C_UINT}),
  xgfxPrimitivesSetFontRotation = define_c_proc(sdl2_gfx, "gfxPrimitivesSetFontRotation", {C_UINT}),
  xcharacterColor = define_c_func(sdl2_gfx, "characterColor", {C_POINTER, C_SHORT, C_SHORT, C_CHAR, C_UINT}, C_INT),
  xcharacterRGBA = define_c_func(sdl2_gfx, "characterRGBA", {C_POINTER, C_SHORT, C_SHORT, C_CHAR, C_UBYTE, C_UBYTE, C_UBYTE,
                                                   C_UBYTE}, C_INT),
  xstringColor = define_c_func(sdl2_gfx, "stringColor", {C_POINTER, C_SHORT, C_SHORT, C_POINTER, C_UINT}, C_INT),
  xstringRGBA = define_c_func(sdl2_gfx, "stringRGBA", {C_POINTER, C_SHORT, C_SHORT, C_POINTER, C_UBYTE, C_UBYTE, C_UBYTE,
                                                   C_UBYTE}, C_INT),

  /*

  Rotozoom functions

  */

  xrotozoomSurface = define_c_func(sdl2_gfx, "rotozoomSurface", {C_POINTER, C_DOUBLE, C_DOUBLE, C_INT}, C_POINTER),
  xrotozoomSurfaceXY = define_c_func(sdl2_gfx, "rotozoomSurfaceXY", {C_POINTER, C_DOUBLE, C_DOUBLE, C_DOUBLE, C_INT},
                                                   C_POINTER),


  xrotozoomSurfaceSize = define_c_proc(sdl2_gfx, "rotozoomSurfaceSize", {C_INT, C_INT, C_DOUBLE, C_DOUBLE, C_POINTER,
                                                   C_POINTER}),
  xrotozoomSurfaceSizeXY = define_c_proc(sdl2_gfx, "rotozoomSurfaceSizeXY", {C_INT, C_INT, C_DOUBLE, C_DOUBLE, C_DOUBLE,
                                                   C_POINTER, C_POINTER}),

  /*

  Zooming functions

  */

  xzoomSurface = define_c_func(sdl2_gfx, "zoomSurface", {C_POINTER, C_DOUBLE, C_DOUBLE, C_INT}, C_POINTER),
  xzoomSurfaceSize = define_c_proc(sdl2_gfx, "zoomSurfaceSize", {C_INT, C_INT, C_DOUBLE, C_DOUBLE, C_POINTER, C_POINTER}),

  /*

  Shrinking functions

  */

  xshrinkSurface = define_c_func(sdl2_gfx, "shrinkSurface", {C_POINTER, C_INT, C_INT}, C_POINTER),

  /*

  Specialized rotation functions

  */

  xrotateSurface90Degrees = define_c_func(sdl2_gfx, "rotateSurface90Degrees", {C_POINTER, C_INT}, C_POINTER),

  /* Framerate functions */

  xSDL_initFramerate  = define_c_proc(sdl2_gfx, "SDL_initFramerate", {C_POINTER}),
  xSDL_setFramerate   = define_c_func(sdl2_gfx, "SDL_setFramerate", {C_POINTER, C_UINT}, C_INT),
  xSDL_getFramerate   = define_c_func(sdl2_gfx, "SDL_getFramerate", {C_POINTER}, C_INT),
  xSDL_getFramecount  = define_c_func(sdl2_gfx, "SDL_getFramecount", {C_POINTER}, C_INT),
  xSDL_framerateDelay = define_c_func(sdl2_gfx, "SDL_framerateDelay", {C_POINTER}, C_UINT),

  /* Image filter functions */

  -- Detect MMX capability in CPU
  xSDL_imageFilterMMXdetect = define_c_func(sdl2_gfx, "SDL_imageFilterMMXdetect", {}, C_INT),

  -- Force use of MMX off (or turn possible use back on)
  xSDL_imageFilterMMXoff = define_c_proc(sdl2_gfx, "SDL_imageFilterMMXoff", {}),
  xSDL_imageFilterMMXon = define_c_proc(sdl2_gfx, "SDL_imageFilterMMXon", {}),

  --  SDL_imageFilterAdd: D = saturation255(S1 + S2)
  xSDL_imageFilterAdd = define_c_func(sdl2_gfx, "SDL_imageFilterAdd", {C_POINTER, C_POINTER, C_POINTER, C_UINT}, C_INT),

  --  SDL_imageFilterMean: D = S1/2 + S2/2
  xSDL_imageFilterMean = define_c_func(sdl2_gfx, "SDL_imageFilterMean", {C_POINTER, C_POINTER, C_POINTER, C_UINT}, C_INT),

  --  SDL_imageFilterSub: D = saturation0(S1 - S2)
  xSDL_imageFilterSub = define_c_func(sdl2_gfx, "SDL_imageFilterSub", {C_POINTER, C_POINTER, C_POINTER, C_UINT}, C_INT),

  --  SDL_imageFilterAbsDiff: D = | S1 - S2 |
  xSDL_imageFilterAbsDiff = define_c_func(sdl2_gfx, "SDL_imageFilterAbsDiff", {C_POINTER, C_POINTER, C_POINTER, C_UINT},
                                                   C_INT),

  --  SDL_imageFilterMult: D = saturation(S1 * S2)
  xSDL_imageFilterMult = define_c_func(sdl2_gfx, "SDL_imageFilterMult", {C_POINTER, C_POINTER, C_POINTER, C_UINT}, C_INT),

  --  SDL_imageFilterMultNor: D = S1 * S2   (non-MMX)
  xSDL_imageFilterMultNor = define_c_func(sdl2_gfx, "SDL_imageFilterMultNor", {C_POINTER, C_POINTER, C_POINTER, C_UINT},
                                                   C_INT),

  --  SDL_imageFilterMultDivby2: D = saturation255(S1/2 * S2)
  xSDL_imageFilterMultDivby2 = define_c_func(sdl2_gfx, "SDL_imageFilterMultDivby2", {C_POINTER, C_POINTER, C_POINTER,
                                                   C_UINT}, C_INT),

  --  SDL_imageFilterMultDivby4: D = saturation255(S1/2 * S2/2)
  xSDL_imageFilterMultDivby4 = define_c_func(sdl2_gfx, "SDL_imageFilterMultDivby4", {C_POINTER, C_POINTER, C_POINTER,
                                                   C_UINT}, C_INT),

  --  SDL_imageFilterBitAnd: D = S1 & S2
  xSDL_imageFilterBitAnd = define_c_func(sdl2_gfx, "SDL_imageFilterBitAnd", {C_POINTER, C_POINTER, C_POINTER, C_UINT},
                                                   C_INT),

  --  SDL_imageFilterBitOr: D = S1 | S2
  xSDL_imageFilterBitOr = define_c_func(sdl2_gfx, "SDL_imageFilterBitOr", {C_POINTER, C_POINTER, C_POINTER, C_UINT},
                                                   C_INT),

  --  SDL_imageFilterDiv: D = S1 / S2   (non-MMX)
  xSDL_imageFilterDiv = define_c_func(sdl2_gfx, "SDL_imageFilterDiv", {C_POINTER, C_POINTER, C_POINTER, C_UINT}, C_INT),

  --  SDL_imageFilterBitNegation: D = !S
  xSDL_imageFilterBitNegation = define_c_func(sdl2_gfx, "SDL_imageFilterBitNegation", {C_POINTER, C_POINTER, C_UINT},
                                                   C_INT),

  --  SDL_imageFilterAddByte: D = saturation255(S + C)
  xSDL_imageFilterAddByte = define_c_func(sdl2_gfx, "SDL_imageFilterAddByte", {C_POINTER, C_POINTER, C_UINT, C_UCHAR},
                                                   C_INT),

  --  SDL_imageFilterAddUint: D = saturation255(S + (uint)C)
  xSDL_imageFilterAddUint = define_c_func(sdl2_gfx, "SDL_imageFilterAddUint", {C_POINTER, C_POINTER, C_UINT, C_UINT},
                                                   C_INT),

  --  SDL_imageFilterAddByteToHalf: D = saturation255(S/2 + C)
  xSDL_imageFilterAddByteToHalf = define_c_func(sdl2_gfx, "SDL_imageFilterAddByteToHalf", {C_POINTER, C_POINTER, C_UINT,
                                                   C_UCHAR}, C_INT),

  --  SDL_imageFilterSubByte: D = saturation0(S - C)
  xSDL_imageFilterSubByte = define_c_func(sdl2_gfx, "SDL_imageFilterSubByte", {C_POINTER, C_POINTER, C_UINT, C_UCHAR},
                                                   C_INT),

  --  SDL_imageFilterSubUint: D = saturation0(S - (uint)C)
  xSDL_imageFilterSubUint = define_c_func(sdl2_gfx, "SDL_imageFilterSubUint", {C_POINTER, C_POINTER, C_UINT, C_UINT},
                                                   C_INT),

  --  SDL_imageFilterShiftRight: D = saturation0(S >> N)
  xSDL_imageFilterShiftRight = define_c_func(sdl2_gfx, "SDL_imageFilterShiftRight", {C_POINTER, C_POINTER, C_UINT, C_UCHAR},
                                                   C_INT),

  --  SDL_imageFilterShiftRightUint: D = saturation0((uint)S >> N)
  xSDL_imageFilterShiftRightUint = define_c_func(sdl2_gfx, "SDL_imageFilterShiftRightUint", {C_POINTER, C_POINTER, C_UINT,
                                                   C_UCHAR}, C_INT),

  --  SDL_imageFilterMultByByte: D = saturation255(S * C)
  xSDL_imageFilterMultByByte = define_c_func(sdl2_gfx, "SDL_imageFilterMultByByte", {C_POINTER, C_POINTER, C_UINT, C_UCHAR},
                                                   C_INT),

  --  SDL_imageFilterShiftRightAndMultByByte: D = saturation255((S >> N) * C)
  xSDL_imageFilterShiftRightAndMultByByte = define_c_func(sdl2_gfx, "SDL_imageFilterShiftRightAndMultByByte", {C_POINTER,
                                                   C_POINTER, C_UINT, C_UCHAR, C_UCHAR}, C_INT),

  --  SDL_imageFilterShiftLeftByte: D = (S << N)
  xSDL_imageFilterShiftLeftByte = define_c_func(sdl2_gfx, "SDL_imageFilterShiftLeftByte", {C_POINTER, C_POINTER, C_UINT,
                                                   C_UCHAR}, C_INT),

  --  SDL_imageFilterShiftLeftUint: D = ((uint)S << N)
  xSDL_imageFilterShiftLeftUint = define_c_func(sdl2_gfx, "SDL_imageFilterShiftLeftUint", {C_POINTER, C_POINTER, C_UINT,
                                                   C_UCHAR}, C_INT),

  --  SDL_imageFilterShiftLeft: D = saturation255(S << N)
  xSDL_imageFilterShiftLeft = define_c_func(sdl2_gfx, "SDL_imageFilterShiftLeft", {C_POINTER, C_POINTER, C_UINT, C_UCHAR},
                                                   C_INT),

  --  SDL_imageFilterBinarizeUsingThreshold: D = S >= T ? 255:0
  xSDL_imageFilterBinarizeUsingThreshold = define_c_func(sdl2_gfx, "SDL_imageFilterBinarizeUsingThreshold", {C_POINTER,
                                                   C_POINTER, C_UINT, C_UCHAR}, C_INT),

  --  SDL_imageFilterClipToRange: D = (S >= Tmin) & (S <= Tmax) 255:0
  xSDL_imageFilterClipToRange = define_c_func(sdl2_gfx, "SDL_imageFilterClipToRange", {C_POINTER, C_POINTER, C_UINT,
                                                   C_UCHAR, C_UCHAR}, C_INT),

  --  SDL_imageFilterNormalizeLinear: D = saturation255((Nmax - Nmin)/(Cmax - Cmin)*(S - Cmin) + Nmin)
  xSDL_imageFilterNormalizeLinear = define_c_func(sdl2_gfx, "SDL_imageFilterNormalizeLinear", {C_POINTER, C_POINTER,
                                                   C_UINT, C_INT, C_INT, C_INT, C_INT}, C_INT)

  /* Ends C function definitions when using C++ */

  /* Pixel */

public function pixelColor(atom renderer, integer x, integer y, integer color)
  return c_func(xpixelColor, {renderer, x, y, color})
end function

public function pixelRGBA(atom renderer, integer x, integer y, integer r, integer g, integer b, integer a)
  return c_func(xpixelRGBA, {renderer, x, y, r, g, b, a})
end function


  /* Horizontal line */

public function hlineColor(atom renderer, integer x1, integer x2, integer y, integer color)
  return c_func(xhlineColor, {renderer, x1, x2, y, color})
end function

public function hlineRGBA(atom renderer, integer x1, integer x2, integer y, integer r, integer g, integer b, integer a)
  return c_func(xhlineRGBA, {renderer, x1, x2, y, r, g, b, a})
end function


  /* Vertical line */

public function vlineColor(atom renderer, integer x, integer y1, integer y2, integer color)
  return c_func(xvlineColor, {renderer, x, y1, y2, color})
end function

public function vlineRGBA(atom renderer, integer x, integer y1, integer y2, integer r, integer g, integer b, integer a)
  return c_func(xvlineRGBA, {renderer, x, y1, y2, r, g, b, a})
end function


  /* Rectangle */

public function rectangleColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer color)
  return c_func(xrectangleColor, {renderer, x1, y1, x2, y2, color})
end function

public function rectangleRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer r, integer g,
                                                   integer b, integer a)
  return c_func(xrectangleRGBA, {renderer, x1, y1, x2, y2, r, g, b, a})
end function


  /* Rounded-Corner Rectangle */

public function roundedRectangleColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer rad,
                                                   integer color)
  return c_func(xroundedRectangleColor, {renderer, x1, y1, x2, y2, rad, color})
end function

public function roundedRectangleRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer rad,
                                                   integer r, integer g, integer b, integer a)
  return c_func(xroundedRectangleRGBA, {renderer, x1, y1, x2, y2, rad, r, g, b, a})
end function


  /* Filled rectangle (Box) */

public function boxColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer color)
  return c_func(xboxColor, {renderer, x1, y1, x2, y2, color})
end function

public function boxRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xboxRGBA, {renderer, x1, y1, x2, y2, r, g, b, a})
end function


  /* Rounded-Corner Filled rectangle (Box) */

public function roundedBoxColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer rad,
                                                   integer color)
  return c_func(xroundedBoxColor, {renderer, x1, y1, x2, y2, rad, color})
end function

public function roundedBoxRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer rad,
                                                   integer r, integer g, integer b, integer a)
  return c_func(xroundedBoxRGBA, {renderer, x1, y1, x2, y2, rad, r, g, b, a})
end function


  /* Line */

public function lineColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer color)
  return c_func(xlineColor, {renderer, x1, y1, x2, y2, color})
end function

public function lineRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer r, integer g,
                                                   integer b, integer a)
  return c_func(xlineRGBA, {renderer, x1, y1, x2, y2, r, g, b, a})
end function


  /* AA Line */

public function aalineColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer color)
  return c_func(xaalineColor, {renderer, x1, y1, x2, y2, color})
end function

public function aalineRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer r, integer g,
                                                   integer b, integer a)
  return c_func(xaalineRGBA, {renderer, x1, y1, x2, y2, r, g, b, a})
end function


  /* Thick Line */
public function thickLineColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer width,
                                                   integer color)
  return c_func(xthickLineColor, {renderer, x1, y1, x2, y2, width, color})
end function

public function thickLineRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer width,
                                                   integer r, integer g, integer b, integer a)
  return c_func(xthickLineRGBA, {renderer, x1, y1, x2, y2, width, r, g, b, a})
end function


  /* Circle */

public function circleColor(atom renderer, integer x, integer y, integer rad, integer color)
  return c_func(xcircleColor, {renderer, x, y, rad, color})
end function

public function circleRGBA(atom renderer, integer x, integer y, integer rad, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xcircleRGBA, {renderer, x, y, rad, r, g, b, a})
end function


  /* Arc */

public function arcColor(atom renderer, integer x, integer y, integer rad, integer start, integer upto,
                                                   integer color)
  return c_func(xarcColor, {renderer, x, y, rad, start, upto, color})
end function

public function arcRGBA(atom renderer, integer x, integer y, integer rad, integer start, integer upto, integer r,
                                                   integer g, integer b, integer a)
  return c_func(xarcRGBA, {renderer, x, y, rad, start, upto, r, g, b, a})
end function


  /* AA Circle */

public function aacircleColor(atom renderer, integer x, integer y, integer rad, integer color)
  return c_func(xaacircleColor, {renderer, x, y, rad, color})
end function

public function aacircleRGBA(atom renderer, integer x, integer y, integer rad, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xaacircleRGBA, {renderer, x, y, rad, r, g, b, a})
end function


  /* Filled Circle */

public function filledCircleColor(atom renderer, integer x, integer y, integer r, integer color)
  return c_func(xfilledCircleColor, {renderer, x, y, r, color})
end function

public function filledCircleRGBA(atom renderer, integer x, integer y, integer rad, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xfilledCircleRGBA, {renderer, x, y, rad, r, g, b, a})
end function


  /* Ellipse */

public function ellipseColor(atom renderer, integer x, integer y, integer rx, integer ry, integer color)
  return c_func(xellipseColor, {renderer, x, y, rx, ry, color})
end function

public function ellipseRGBA(atom renderer, integer x, integer y, integer rx, integer ry, integer r, integer g,
                                                   integer b, integer a)
  return c_func(xellipseRGBA, {renderer, x, y, rx, ry, r, g, b, a})
end function


  /* AA Ellipse */

public function aaellipseColor(atom renderer, integer x, integer y, integer rx, integer ry, integer color)
  return c_func(xaaellipseColor, {renderer, x, y, rx, ry, color})
end function

public function aaellipseRGBA(atom renderer, integer x, integer y, integer rx, integer ry, integer r, integer g,
                                                   integer b, integer a)
  return c_func(xaaellipseRGBA, {renderer, x, y, rx, ry, r, g, b, a})
end function


  /* Filled Ellipse */

public function filledEllipseColor(atom renderer, integer x, integer y, integer rx, integer ry, integer color)
  return c_func(xfilledEllipseColor, {renderer, x, y, rx, ry, color})
end function

public function filledEllipseRGBA(atom renderer, integer x, integer y, integer rx, integer ry, integer r, integer g,
                                                   integer b, integer a)
  return c_func(xfilledEllipseRGBA, {renderer, x, y, rx, ry, r, g, b, a})
end function


  /* Pie */

public function pieColor(atom renderer, integer x, integer y, integer rad, integer start, integer upto,
                                                   integer color)
  return c_func(xpieColor, {renderer, x, y, rad, start, upto, color})
end function

public function pieRGBA(atom renderer, integer x, integer y, integer rad, integer start, integer upto, integer r,
                                                   integer g, integer b, integer a)
  return c_func(xpieRGBA, {renderer, x, y, rad, start, upto, r, g, b, a})
end function


  /* Filled Pie */

public function filledPieColor(atom renderer, integer x, integer y, integer rad, integer start, integer upto,
                                                   integer color)
  return c_func(xfilledPieColor, {renderer, x, y, rad, start, upto, color})
end function

public function filledPieRGBA(atom renderer, integer x, integer y, integer rad, integer start, integer upto,
                                                   integer r, integer g, integer b, integer a)
  return c_func(xfilledPieRGBA, {renderer, x, y, rad, start, upto, r, g, b, a})
end function


  /* Trigon */

public function trigonColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer x3, integer y3,
                                                   integer color)
  return c_func(xtrigonColor, {renderer, x1, y1, x2, y2, x3, y3, color})
end function

public function trigonRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer x3, integer y3,
                                                   integer r, integer g, integer b, integer a)
  return c_func(xtrigonRGBA, {renderer, x1, y1, x2, y2, x3, y3, r, g, b, a})
end function


  /* AA-Trigon */

public function aatrigonColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer x3,
                                                   integer y3, integer color)
  return c_func(xaatrigonColor, {renderer, x1, y1, x2, y2, x3, y3, color})
end function

public function aatrigonRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer x3,
                                                   integer y3, integer r, integer g, integer b, integer a)
  return c_func(xaatrigonRGBA, {renderer, x1, y1, x2, y2, x3, y3, r, g, b, a})
end function


  /* Filled Trigon */

public function filledTrigonColor(atom renderer, integer x1, integer y1, integer x2, integer y2, integer x3,
                                                   integer y3, integer color)
  return c_func(xfilledTrigonColor, {renderer, x1, y1, x2, y2, x3, y3, color})
end function

public function filledTrigonRGBA(atom renderer, integer x1, integer y1, integer x2, integer y2, integer x3,
                                                   integer y3, integer r, integer g, integer b, integer a)
  return c_func(xfilledTrigonRGBA, {renderer, x1, y1, x2, y2, x3, y3, r, g, b, a})
end function


  /* Polygon */

public function polygonColor(atom renderer, atom vx, atom vy, integer n, integer color)
  return c_func(xpolygonColor, {renderer, vx, vy, n, color})
end function

public function polygonRGBA(atom renderer, atom vx, atom vy, integer n, integer r, integer g, integer b, integer a)
  return c_func(xpolygonRGBA, {renderer, vx, vy, n, r, g, b, a})
end function


  /* AA-Polygon */

public function aapolygonColor(atom renderer, atom vx, atom vy, integer n, integer color)
  return c_func(xaapolygonColor, {renderer, vx, vy, n, color})
end function

public function aapolygonRGBA(atom renderer, atom vx, atom vy, integer n, integer r, integer g, integer b, integer a)
  return c_func(xaapolygonRGBA, {renderer, vx, vy, n, r, g, b, a})
end function


  /* Filled Polygon */

public function filledPolygonColor(atom renderer, atom vx, atom vy, integer n, integer color)
  return c_func(xfilledPolygonColor, {renderer, vx, vy, n, color})
end function

public function filledPolygonRGBA(atom renderer, atom vx, atom vy, integer n, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xfilledPolygonRGBA, {renderer, vx, vy, n, r, g, b, a})
end function


  /* Textured Polygon */

public function texturedPolygon(atom renderer, atom vx, atom vy, integer n, atom texture, integer texture_dx,
                                                   integer texture_dy)
  return c_func(xtexturedPolygon, {renderer, vx, vy, n, texture, texture_dx, texture_dy})
end function


  /* Bezier */

public function bezierColor(atom renderer, atom vx, atom vy, integer n, integer s, integer color)
  return c_func(xbezierColor, {renderer, vx, vy, n, s, color})
end function

public function bezierRGBA(atom renderer, atom vx, atom vy, integer n, integer s, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xbezierRGBA, {renderer, vx, vy, n, s, r, g, b, a})
end function


  /* Characters/Strings */

public procedure gfxPrimitivesSetFont(atom fontdata, integer cw, integer ch)
  c_proc(xgfxPrimitivesSetFont, {fontdata, cw, ch})
end procedure

public procedure gfxPrimitivesSetFontRotation(integer rotation)
  c_proc(xgfxPrimitivesSetFontRotation, {rotation})
end procedure

public function characterColor(atom renderer, integer x, integer y, integer c, integer color)
  return c_func(xcharacterColor, {renderer, x, y, c, color})
end function

public function characterRGBA(atom renderer, integer x, integer y, integer c, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xcharacterRGBA, {renderer, x, y, c, r, g, b, a})
end function

public function stringColor(atom renderer, integer x, integer y, atom s, integer color)
  return c_func(xstringColor, {renderer, x, y, s, color})
end function

public function stringRGBA(atom renderer, integer x, integer y, atom s, integer r, integer g, integer b,
                                                   integer a)
  return c_func(xstringRGBA, {renderer, x, y, s, r, g, b, a})
end function

  /*

  Rotozoom functions

  */

public function rotozoomSurface(atom renderer, atom angle, atom zoom, integer smooth)
  return c_func(xrotozoomSurface, {renderer, angle, zoom, smooth})
end function

public function rotozoomSurfaceXY(atom renderer, atom angle, atom zoomx, atom zoomy, integer smooth)
  return c_func(xrotozoomSurfaceXY, {renderer, angle, zoomx, zoomy, smooth})
end function


public procedure rotozoomSurfaceSize(integer width, integer height, atom angle, atom zoom, atom dstwidth,
                                                   atom dstheight)
  c_proc(xrotozoomSurfaceSize, {width, height, angle, zoom, dstwidth, dstheight})
end procedure

public procedure rotozoomSurfaceSizeXY(integer width, integer height, atom angle, atom zoomx, atom zoomy,
                                                   atom dstwidth, atom dstheight)
  c_proc(xrotozoomSurfaceSizeXY, {width, height, angle, zoomx, zoomy, dstwidth, dstheight})
end procedure

  /*

  Zooming functions

  */

public function zoomSurface(atom renderer, atom zoomx, atom zoomy, integer smooth)
  return c_func(xzoomSurface, {renderer, zoomx, zoomy, smooth})
end function

public procedure zoomSurfaceSize(integer width, integer height, atom zoomx, atom zoomy, atom dstwidth, atom dstheight)
  c_proc(xzoomSurfaceSize, {width, height, zoomx, zoomy, dstwidth, dstheight})
end procedure

  /*

  Shrinking functions

  */

public function shrinkSurface(atom renderer, integer factorx, integer factory)
  return c_func(xshrinkSurface, {renderer, factorx, factory})
end function

  /*

  Specialized rotation functions

  */

public function rotateSurface90Degrees(atom renderer, integer numClockwiseTurns)
  return c_func(xrotateSurface90Degrees, {renderer, numClockwiseTurns})
end function

  /* Framerate functions */

  /* Functions return 0 or value for success and -1 for error */

public procedure SDL_initFramerate(atom manager)
  c_proc(xSDL_initFramerate, {manager})
end procedure

public function SDL_setFramerate(atom manager, atom rate)
  return c_func(xSDL_setFramerate, {manager, rate})
end function

public function SDL_getFramerate(atom manager)
  return c_func(xSDL_getFramerate, {manager})
end function

public function SDL_getFramecount(atom manager)
  return c_func(xSDL_getFramecount, {manager})
end function

public function SDL_framerateDelay(atom manager)
  return c_func(xSDL_framerateDelay, {manager})
end function

  /* Image filter functions */

  /* Comments:                                                                           */
  /*  1.) MMX functions work best if all data blocks are aligned on a 32 bytes boundary. */
  /*  2.) Data that is not within an 8 byte boundary is processed using the C routine.   */
  /*  3.) Convolution routines do not have C routines at this time.                      */

-- Detect MMX capability in CPU
public function SDL_imageFilterMMXdetect()
  return c_func(xSDL_imageFilterMMXdetect, {})
end function

-- Force use of MMX off (or turn possible use back on)
public procedure SDL_imageFilterMMXoff()
  c_proc(xSDL_imageFilterMMXoff, {})
end procedure

public procedure SDL_imageFilterMMXon()
  c_proc(xSDL_imageFilterMMXon, {})
end procedure

  --
  -- All routines return:
  --   0   OK
  --  -1   Error (internal error, parameter error)
  --

  --  SDL_imageFilterAdd: D = saturation255(S1 + S2)
public function SDL_imageFilterAdd(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterAdd, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterMean: D = S1/2 + S2/2
public function SDL_imageFilterMean(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterMean, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterSub: D = saturation0(S1 - S2)
public function SDL_imageFilterSub(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterSub, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterAbsDiff: D = | S1 - S2 |
public function SDL_imageFilterAbsDiff(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterAbsDiff, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterMult: D = saturation(S1 * S2)
public function SDL_imageFilterMult(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterMult, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterMultNor: D = S1 * S2   (non-MMX)
public function SDL_imageFilterMultNor(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterMultNor, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterMultDivby2: D = saturation255(S1/2 * S2)
public function SDL_imageFilterMultDivby2(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterMultDivby2, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterMultDivby4: D = saturation255(S1/2 * S2/2)
public function SDL_imageFilterMultDivby4(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterMultDivby4, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterBitAnd: D = S1 & S2
public function SDL_imageFilterBitAnd(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterBitAnd, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterBitOr: D = S1 | S2
public function SDL_imageFilterBitOr(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterBitOr, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterDiv: D = S1 / S2   (non-MMX)
public function SDL_imageFilterDiv(atom Src1, atom Src2, atom Dest, integer lg)
  return c_func(xSDL_imageFilterDiv, {Src1, Src2, Dest, lg})
end function

--  SDL_imageFilterBitNegation: D = !S
public function SDL_imageFilterBitNegation(atom Src1, atom Dest, integer lg)
  return c_func(xSDL_imageFilterBitNegation, {Src1, Dest, lg})
end function

--  SDL_imageFilterAddByte: D = saturation255(S + C)
public function SDL_imageFilterAddByte(atom Src1, atom Dest, integer lg, integer C)
  return c_func(xSDL_imageFilterAddByte, {Src1, Dest, lg, C})
end function

--  SDL_imageFilterAddUint: D = saturation255(S + (uint)C)
public function SDL_imageFilterAddUint(atom Src1, atom Dest, integer lg, integer C)
  return c_func(xSDL_imageFilterAddUint, {Src1, Dest, lg, C})
end function

--  SDL_imageFilterAddByteToHalf: D = saturation255(S/2 + C)
public function SDL_imageFilterAddByteToHalf(atom Src1, atom Dest, integer lg, integer C)
  return c_func(xSDL_imageFilterAddByteToHalf, {Src1, Dest, lg, C})
end function

--  SDL_imageFilterSubByte: D = saturation0(S - C)
public function SDL_imageFilterSubByte(atom Src1, atom Dest, integer lg, integer C)
  return c_func(xSDL_imageFilterSubByte, {Src1, Dest, lg, C})
end function

--  SDL_imageFilterSubUint: D = saturation0(S - (uint)C)
public function SDL_imageFilterSubUint(atom Src1, atom Dest, integer lg, integer C)
  return c_func(xSDL_imageFilterSubUint, {Src1, Dest, lg, C})
end function

--  SDL_imageFilterShiftRight: D = saturation0(S >> N)
public function SDL_imageFilterShiftRight(atom Src1, atom Dest, integer lg, integer N)
  return c_func(xSDL_imageFilterShiftRight, {Src1, Dest, lg, N})
end function

--  SDL_imageFilterShiftRightUint: D = saturation0((uint)S >> N)
public function SDL_imageFilterShiftRightUint(atom Src1, atom Dest, integer lg, integer N)
  return c_func(xSDL_imageFilterShiftRightUint, {Src1, Dest, lg, N})
end function

--  SDL_imageFilterMultByByte: D = saturation255(S * C)
public function SDL_imageFilterMultByByte(atom Src1, atom Dest, integer lg, integer C)
  return c_func(xSDL_imageFilterMultByByte, {Src1, Dest, lg, C})
end function

--  SDL_imageFilterShiftRightAndMultByByte: D = saturation255((S >> N) * C)
public function SDL_imageFilterShiftRightAndMultByByte(atom Src1, atom Dest, integer lg, integer N, integer C)
  return c_func(xSDL_imageFilterShiftRightAndMultByByte, {Src1, Dest, lg, N, C})
end function

--  SDL_imageFilterShiftLeftByte: D = (S << N)
public function SDL_imageFilterShiftLeftByte(atom Src1, atom Dest, integer lg, integer N)
  return c_func(xSDL_imageFilterShiftLeftByte, {Src1, Dest, lg, N})
end function

--  SDL_imageFilterShiftLeftUint: D = ((uint)S << N)
public function SDL_imageFilterShiftLeftUint(atom Src1, atom Dest, integer lg, integer N)
  return c_func(xSDL_imageFilterShiftLeftUint, {Src1, Dest, lg, N})
end function

--  SDL_imageFilterShiftLeft: D = saturation255(S << N)
public function SDL_imageFilterShiftLeft(atom Src1, atom Dest, integer lg, integer N)
  return c_func(xSDL_imageFilterShiftLeft, {Src1, Dest, lg, N})
end function

--  SDL_imageFilterBinarizeUsingThreshold: D = S >= T ? 255:0
public function SDL_imageFilterBinarizeUsingThreshold(atom Src1, atom Dest, integer lg, integer T)
  return c_func(xSDL_imageFilterBinarizeUsingThreshold, {Src1, Dest, lg, T})
end function

--  SDL_imageFilterClipToRange: D = (S >= Tmin) & (S <= Tmax) 255:0
public function SDL_imageFilterClipToRange(atom Src1, atom Dest, integer lg, integer Tmin, integer Tmax)
  return c_func(xSDL_imageFilterClipToRange, {Src1, Dest, lg, Tmin, Tmax})
end function

--  SDL_imageFilterNormalizeLinear: D = saturation255((Nmax - Nmin)/(Cmax - Cmin)*(S - Cmin) + Nmin)
public function SDL_imageFilterNormalizeLinear(atom Src, atom Dest, integer lg, integer Cmin, integer Cmax,
                                                   integer Nmin, integer Nmax)
  return c_func(xSDL_imageFilterNormalizeLinear, {Src, Dest, lg, Cmin, Cmax, Nmin, Nmax})
end function


  /* Ends C function definitions when using C++ */
